function [ac,bc,cc,dc,cost,test] = rolqg(a,b,c,d,d1,d2,e1,e2,e0,nc)
%ROLQG     reduced-order lqg compensator design
%
%          [ac,bc,cc,dc,cost,test] = ROLQG(a,b,c,d,d1,d2,e1,e2,e0,nc)
%          returns the nc-th order H-2 sub-optimal controller.  The
%          controller is a model-reduced version of the full order
%          H-2 optimal controller.  cost is the H-2 cost of the 
%          quadratic cost function
%
%                       _infty
%                      /
%                 J =  |  (x'.e1'.e1.x + u'.e2'.e2.u ) dt
%                     _/
%                     0
%          
%          test is a flag which evaluates the closed-loop stability
%          of the system when the reduced-order compensator is 
%          included in the feedback loop.  If test = 0, the closed-
%          loop is stable, while if test = 1, the closed-loop is
%          unstable, and cost is undefined.
%
%          see also LQR, LQE, REG, BALREAL, MODRED, LYAP
%
[nx,nw] = size(d1);
[np,np] = size(a);
r1 = e1'*e1;
r2 = e2'*e2;
v1 = d1*d1';
v2 = d2*d2';
[k,s,e] = lqr(a,b,r1,r2);
l = lqe(a,d1,c,eye(nw),v2);
[ac,bc,cc,dc] = reg(a,b,c,d,k,l);
cc = -1*cc;
[acb,bcb,ccb] = balreal(ac,bc,cc);
elim = [nc+1:1:np];
[ac,bc,cc,temp] = modred(acb,bcb,ccb,dc,elim);
atil = [a,b*cc;bc*c,ac];
dtil = [d1;bc*d2];
etil = [e1, e2*cc];
rtil = etil'*etil;
vtil = dtil*dtil';
qtil = lyap(atil, vtil);
cost = trace(qtil*rtil);
test = initchk(ac,bc,cc,dc,a,b,c,d,d1,d2,e1,e2,e0);
if test == 1
	disp(' ')
	disp('Closed-loop system unstable')
        disp(' ')
        clear ac bc cc dc cost 
	return
end
